mod queyd;
extern crate slugify;
use actix_web::{guard, web, web::Data, App, HttpResponse, HttpServer, Result};
use async_graphql::{
    http::{playground_source, GraphQLPlaygroundConfig},
    EmptySubscription, Schema,
};
use async_graphql_actix_web::{GraphQLRequest, GraphQLResponse};
use queyd::*;

async fn index(schema: web::Data<QueydSchema>, req: GraphQLRequest) -> GraphQLResponse {
    schema.execute(req.into_inner()).await.into()
}

async fn index_playground() -> Result<HttpResponse> {
    let source =
        playground_source(GraphQLPlaygroundConfig::new("/api").subscription_endpoint("/api"));
    Ok(HttpResponse::Ok()
        .content_type("text/html; charset=utf-8")
        .body(source))
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    let schema = Schema::build(QueryRoot, MutationRoot, EmptySubscription)
        .data(Queyd::new())
        .finish();

    println!("Playground: http://localhost:8000");

    HttpServer::new(move || {
        App::new()
            .app_data(Data::new(schema.clone()))
            .service(web::resource("/api").guard(guard::Post()).to(index))
            .service(
                web::resource("/api-playground")
                    .guard(guard::Get())
                    .to(index_playground),
            )
    })
    .bind("127.0.0.1:8000")?
    .run()
    .await
}
